/*
 * Copyright © 2013-2014 Collabora, Ltd.
 * Copyright © 2016 DENSO CORPORATION
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdint.h>

#include <waltham-object.h>
#include <waltham-util.h>

#ifdef  __cplusplus
extern "C" {
#endif

struct wth_display;
struct wthp_blob_factory;
struct wthp_buffer;
struct wthp_callback;
struct wthp_compositor;
struct wthp_farstream;
struct wthp_farstream_remote;
struct wthp_keyboard;
struct wthp_output;
struct wthp_pointer;
struct wthp_region;
struct wthp_registry;
struct wthp_resource;
struct wthp_seat;
struct wthp_surface;
struct wthp_touch;

#ifndef WALTHAM_SERVER_H
#define WALTHAM_SERVER_H

struct wth_display_interface {
  void (*client_version) (struct wth_display * wth_display, uint32_t client_version);
  void (*sync) (struct wth_display * wth_display, struct wthp_callback * callback);
  void (*get_registry) (struct wth_display * wth_display, struct wthp_registry * registry);
};

static inline void
wth_display_set_interface(struct wth_display *self, const struct wth_display_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_registry_interface {
  void (*destroy) (struct wthp_registry * wthp_registry);
  void (*bind) (struct wthp_registry * wthp_registry, uint32_t name, struct wth_object * id, const char * interface, uint32_t version);
};

static inline void
wthp_registry_set_interface(struct wthp_registry *self, const struct wthp_registry_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_compositor_interface {
  void (*create_surface) (struct wthp_compositor * wthp_compositor, struct wthp_surface * id);
  void (*create_region) (struct wthp_compositor * wthp_compositor, struct wthp_region * id);
};

static inline void
wthp_compositor_set_interface(struct wthp_compositor *self, const struct wthp_compositor_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_blob_factory_interface {
  void (*create_buffer) (struct wthp_blob_factory * wthp_blob_factory, struct wthp_buffer * buffer, uint32_t data_sz, void * data, int32_t width, int32_t height, int32_t stride, uint32_t format);
};

static inline void
wthp_blob_factory_set_interface(struct wthp_blob_factory *self, const struct wthp_blob_factory_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_buffer_interface {
  void (*destroy) (struct wthp_buffer * wthp_buffer);
};

static inline void
wthp_buffer_set_interface(struct wthp_buffer *self, const struct wthp_buffer_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_surface_interface {
  void (*destroy) (struct wthp_surface * wthp_surface);
  void (*attach) (struct wthp_surface * wthp_surface, struct wthp_buffer * buffer, int32_t x, int32_t y);
  void (*damage) (struct wthp_surface * wthp_surface, int32_t x, int32_t y, int32_t width, int32_t height);
  void (*frame) (struct wthp_surface * wthp_surface, struct wthp_callback * callback);
  void (*set_opaque_region) (struct wthp_surface * wthp_surface, struct wthp_region * region);
  void (*set_input_region) (struct wthp_surface * wthp_surface, struct wthp_region * region);
  void (*commit) (struct wthp_surface * wthp_surface);
  void (*set_buffer_transform) (struct wthp_surface * wthp_surface, int32_t transform);
  void (*set_buffer_scale) (struct wthp_surface * wthp_surface, int32_t scale);
  void (*damage_buffer) (struct wthp_surface * wthp_surface, int32_t x, int32_t y, int32_t width, int32_t height);
};

static inline void
wthp_surface_set_interface(struct wthp_surface *self, const struct wthp_surface_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_seat_interface {
  void (*get_pointer) (struct wthp_seat * wthp_seat, struct wthp_pointer * id);
  void (*get_keyboard) (struct wthp_seat * wthp_seat, struct wthp_keyboard * id);
  void (*get_touch) (struct wthp_seat * wthp_seat, struct wthp_touch * id);
  void (*release) (struct wthp_seat * wthp_seat);
};

static inline void
wthp_seat_set_interface(struct wthp_seat *self, const struct wthp_seat_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_pointer_interface {
  void (*set_cursor) (struct wthp_pointer * wthp_pointer, uint32_t serial, struct wthp_surface * surface, int32_t hotspot_x, int32_t hotspot_y);
  void (*release) (struct wthp_pointer * wthp_pointer);
};

static inline void
wthp_pointer_set_interface(struct wthp_pointer *self, const struct wthp_pointer_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_keyboard_interface {
  void (*release) (struct wthp_keyboard * wthp_keyboard);
};

static inline void
wthp_keyboard_set_interface(struct wthp_keyboard *self, const struct wthp_keyboard_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_touch_interface {
  void (*release) (struct wthp_touch * wthp_touch);
};

static inline void
wthp_touch_set_interface(struct wthp_touch *self, const struct wthp_touch_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_region_interface {
  void (*destroy) (struct wthp_region * wthp_region);
  void (*add) (struct wthp_region * wthp_region, int32_t x, int32_t y, int32_t width, int32_t height);
  void (*subtract) (struct wthp_region * wthp_region, int32_t x, int32_t y, int32_t width, int32_t height);
};

static inline void
wthp_region_set_interface(struct wthp_region *self, const struct wthp_region_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_farstream_interface {
  void (*destroy) (struct wthp_farstream * wthp_farstream);
  void (*connect) (struct wthp_farstream * wthp_farstream, uint32_t port, struct wthp_farstream_remote * remote);
};

static inline void
wthp_farstream_set_interface(struct wthp_farstream *self, const struct wthp_farstream_interface *funcs, void *user_data)
{
  wth_object_set_listener((struct wth_object *)self, (void (**)(void)) funcs, user_data);
}

struct wthp_farstream_remote_interface {
  void (*destroy) (struct wthp_farstream_remote * wthp_farstream_remote);
  void (*codec_offer) (struct wthp_farstream_remote * wthp_farstream_remote, const char * list);
  void (*create_buffer) (struct wthp_farstream_remote * wthp_farstream_remote, uint32_t frame_id, struct wthp_buffer * id);
};

static inline void
wth_display_free(struct wth_display *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wth_display_send_error (struct wth_display * wth_display, struct wth_object * object_id, uint32_t code, const char * message);

void
wth_display_send_delete_id (struct wth_display * wth_display, uint32_t id);

void
wth_display_send_server_version (struct wth_display * wth_display, uint32_t server_version);

static inline void
wthp_registry_free(struct wthp_registry *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_registry_send_global (struct wthp_registry * wthp_registry, uint32_t name, const char * interface, uint32_t version);

void
wthp_registry_send_global_remove (struct wthp_registry * wthp_registry, uint32_t name);

static inline void
wthp_callback_free(struct wthp_callback *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_callback_send_done (struct wthp_callback * wthp_callback, uint32_t callback_data);

static inline void
wthp_compositor_free(struct wthp_compositor *self)
{
  wth_object_delete((struct wth_object *)self);
}

static inline void
wthp_blob_factory_free(struct wthp_blob_factory *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_blob_factory_send_format (struct wthp_blob_factory * wthp_blob_factory, uint32_t format);

static inline void
wthp_buffer_free(struct wthp_buffer *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_buffer_send_complete (struct wthp_buffer * wthp_buffer, uint32_t serial);

static inline void
wthp_surface_free(struct wthp_surface *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_surface_send_enter (struct wthp_surface * wthp_surface, struct wthp_output * output);

void
wthp_surface_send_leave (struct wthp_surface * wthp_surface, struct wthp_output * output);

static inline void
wthp_seat_free(struct wthp_seat *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_seat_send_capabilities (struct wthp_seat * wthp_seat, uint32_t capabilities);

void
wthp_seat_send_name (struct wthp_seat * wthp_seat, const char * name);

static inline void
wthp_pointer_free(struct wthp_pointer *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_pointer_send_enter (struct wthp_pointer * wthp_pointer, uint32_t serial, struct wthp_surface * surface, wth_fixed_t surface_x, wth_fixed_t surface_y);

void
wthp_pointer_send_leave (struct wthp_pointer * wthp_pointer, uint32_t serial, struct wthp_surface * surface);

void
wthp_pointer_send_motion (struct wthp_pointer * wthp_pointer, uint32_t time, wth_fixed_t surface_x, wth_fixed_t surface_y);

void
wthp_pointer_send_button (struct wthp_pointer * wthp_pointer, uint32_t serial, uint32_t time, uint32_t button, uint32_t state);

void
wthp_pointer_send_axis (struct wthp_pointer * wthp_pointer, uint32_t time, uint32_t axis, wth_fixed_t value);

void
wthp_pointer_send_frame (struct wthp_pointer * wthp_pointer);

void
wthp_pointer_send_axis_source (struct wthp_pointer * wthp_pointer, uint32_t axis_source);

void
wthp_pointer_send_axis_stop (struct wthp_pointer * wthp_pointer, uint32_t time, uint32_t axis);

void
wthp_pointer_send_axis_discrete (struct wthp_pointer * wthp_pointer, uint32_t axis, int32_t discrete);

static inline void
wthp_keyboard_free(struct wthp_keyboard *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_keyboard_send_keymap (struct wthp_keyboard * wthp_keyboard, uint32_t format, uint32_t keymap_sz, void * keymap);

void
wthp_keyboard_send_enter (struct wthp_keyboard * wthp_keyboard, uint32_t serial, struct wthp_surface * surface, struct wth_array * keys);

void
wthp_keyboard_send_leave (struct wthp_keyboard * wthp_keyboard, uint32_t serial, struct wthp_surface * surface);

void
wthp_keyboard_send_key (struct wthp_keyboard * wthp_keyboard, uint32_t serial, uint32_t time, uint32_t key, uint32_t state);

void
wthp_keyboard_send_modifiers (struct wthp_keyboard * wthp_keyboard, uint32_t serial, uint32_t mods_depressed, uint32_t mods_latched, uint32_t mods_locked, uint32_t group);

void
wthp_keyboard_send_repeat_info (struct wthp_keyboard * wthp_keyboard, int32_t rate, int32_t delay);

static inline void
wthp_touch_free(struct wthp_touch *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_touch_send_down (struct wthp_touch * wthp_touch, uint32_t serial, uint32_t time, struct wthp_surface * surface, int32_t id, wth_fixed_t x, wth_fixed_t y);

void
wthp_touch_send_up (struct wthp_touch * wthp_touch, uint32_t serial, uint32_t time, int32_t id);

void
wthp_touch_send_motion (struct wthp_touch * wthp_touch, uint32_t time, int32_t id, wth_fixed_t x, wth_fixed_t y);

void
wthp_touch_send_frame (struct wthp_touch * wthp_touch);

void
wthp_touch_send_cancel (struct wthp_touch * wthp_touch);

static inline void
wthp_output_free(struct wthp_output *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_output_send_geometry (struct wthp_output * wthp_output, int32_t x, int32_t y, int32_t physical_width, int32_t physical_height, int32_t subpixel, const char * make, const char * model, int32_t transform);

void
wthp_output_send_mode (struct wthp_output * wthp_output, uint32_t flags, int32_t width, int32_t height, int32_t refresh);

void
wthp_output_send_done (struct wthp_output * wthp_output);

void
wthp_output_send_scale (struct wthp_output * wthp_output, int32_t factor);

static inline void
wthp_region_free(struct wthp_region *self)
{
  wth_object_delete((struct wth_object *)self);
}

static inline void
wthp_farstream_free(struct wthp_farstream *self)
{
  wth_object_delete((struct wth_object *)self);
}

static inline void
wthp_farstream_remote_free(struct wthp_farstream_remote *self)
{
  wth_object_delete((struct wth_object *)self);
}

void
wthp_farstream_remote_send_port (struct wthp_farstream_remote * wthp_farstream_remote, uint32_t port);

void
wthp_farstream_remote_send_codec_answer (struct wthp_farstream_remote * wthp_farstream_remote, const char * list);

void
wthp_farstream_remote_send_error (struct wthp_farstream_remote * wthp_farstream_remote, uint32_t code, const char * message);

#ifdef  __cplusplus
}
#endif

#endif